# Microeconometrics
# CLASS 9d
# Dynamic panel models
# Wiktor Budzinski

library(splm)
library(plm)
library(readxl)
library(stargazer)
library(sandwich)
library(lmtest)
library(nlme)
library(zoo)
library(pder)

rm(list=ls())

data("RiceFarms", package = "splm")
Rice <- pdata.frame(RiceFarms, index = "id")
# Transforming data
Rice$logoutput <- log(Rice$goutput) # gross output of rice in kg
Rice$logseed <- log(Rice$seed) # seed in kilogram
Rice$loglabor <-   log(Rice$totlabor) # total labor (excluding harvest labor)
Rice$logsize <-   log(Rice$size) # the total area cultivated with rice, measured in hectares

Eq1 <- logoutput~logseed+loglabor+logsize

# Fixed effect model for comparison
fe <- plm(Eq1, Rice, model = "within")
summary(fe)

# Dynamic AR(1) model
Eq2 <- logoutput~ lag(logoutput,1) + logseed+loglabor+logsize | lag(logoutput,2:6)
gmm1 <- pgmm(Eq2, data = Rice, model = "onestep")
summary(gmm1)
coeftest(gmm1, vcov.=vcovHC(gmm1)) 

# Two-step method
gmm2 <- pgmm(Eq2, data = Rice, model = "twosteps")
summary(gmm2)
coeftest(gmm2,  vcov.=vcovHC(gmm2)) 
summary(gmm2, robust =FALSE)

# Adding Xs as additional instruments does not really change anything
Eq2b <- logoutput~ lag(logoutput,1) + logseed+loglabor+logsize | lag(logoutput,2:6) | logseed+loglabor+logsize
gmm2b <- pgmm(Eq2b, data = Rice, model = "twosteps")
summary(gmm2b)

# System - GMM
gmmsys <- pgmm(Eq2, data = Rice, model = "twosteps", effect = "twoways", transformation = "ld")
summary(gmmsys)