# Microeconometrics
# CLASS 9d
# Dynamic panel data
# Wiktor Budzinski

rm(list=ls())
library(plm)
library(readxl)
library(stargazer)
library(sandwich)
library(lmtest)
library(nlme)
library(zoo)
library(ramify)
library(tidyr)
library(mvtnorm)

############################ Exercise 1 ##################################

# Simulating dynamic panel data
  # Random effects
  # x1, x2 are not autocorrelated

# 500 observations
# 8 periods (+ 0 period)

rm(list=ls())
set.seed(1234567) # setting seed

rho<-0.5 # Autoregressive term
# Generating ID
id<- c(t(kronecker(matrix(1,1,8),1:500)))
# Independent variables
x1 <- rnorm(4500,0,2)
x2 <- rnorm(4500,0,1)

Fit <- x1*0.5 -x2 # XB
Fit <-matrix(Fit, nrow = 9, ncol = 500)
A <-diag(9) 
A[2:9,1:8] <- A[2:9,1:8]-rho*diag(8)
Mat <- inv(A)
SIG<- diag(9)*1.25
SIG[SIG == 0] <- 0.25
VAR<- Mat %*% SIG %*% t(Mat) # Variance

Means <- Mat %*% Fit # Means

y <- matrix(, nrow = 500, ncol = 9) # Dependent variable
for(i in 1:500){
  y[i,] <- rmvnorm(n=1, mean=t(Means[,i]), sigma=VAR)
}
# Some data transformation
y <- matrix(t(y[,2:9]), nrow = 4000, ncol = 1)
x1 <- matrix(x1, nrow = 9, ncol = 500)
x1 <- matrix(x1[2:9,], nrow = 4000, ncol = 1)
x2 <- matrix(x2, nrow = 9, ncol = 500)
x2 <- matrix(x2[2:9,], nrow = 4000, ncol = 1)

data<-data.frame(id, y,x1, x2)
Panel_data<-pdata.frame(data,index = c("id"))

# Models
# True values: lag(y): 0.5, x1: 0.5, x2: -1, 
m_p <- plm(y ~ x1 + x2, data = Panel_data, model = "pooling")
m_f1 <- plm(y ~ x1 + x2, data = Panel_data, model = "within")
m_f2 <- plm(y ~ lag(y, 1) + x1 + x2, data = Panel_data, model = "within")
m_gmm <- pgmm(y ~ lag(y, 1) + x1 + x2 | lag(y, 2:8), data = Panel_data, model = "twosteps")
summary(m_gmm)
stargazer(m_p, m_f1, m_f2,m_gmm, type="text", df=FALSE)  

############################ Exercise 2 ##################################
# Simulating dynamic panel data
  # Random effects
  # x1, is now autocorrelated

rm(list=ls())
set.seed(1234567) # setting seed

rho<-0.5 # Autoregressive term
# Generating ID
id<- c(t(kronecker(matrix(1,1,8),1:500)))
# Independent variables
#
rho2 <- 0.8
# Generating x1
A <-diag(9) 
A[2:9,1:8] <- A[2:9,1:8]-rho2*diag(8)
Mat <- inv(A)
SIG<- diag(9)
VAR<- Mat %*% SIG %*% t(Mat) # Variance
x1 <- matrix(, nrow = 500, ncol = 9) # Dependent variable
for(i in 1:500){
  x1[i,] <- rmvnorm(n=1, mean=rep(0, 9), sigma=VAR)
}

x11 <-matrix(t(x1), nrow = 4500, ncol = 1)
x2 <- rnorm(4500,0,1)

Fit <- x11*0.5 -x2 # XB
Fit <-matrix(Fit, nrow = 9, ncol = 500)
A <-diag(9) 
A[2:9,1:8] <- A[2:9,1:8]-rho*diag(8)
Mat <- inv(A)
SIG<- diag(9)*1.25
SIG[SIG == 0] <- 0.25
VAR<- Mat %*% SIG %*% t(Mat) # Variance

Means <- Mat %*% Fit # Means

y <- matrix(, nrow = 500, ncol = 9) # Dependent variable
for(i in 1:500){
  y[i,] <- rmvnorm(n=1, mean=t(Means[,i]), sigma=VAR)
}
# Some data transformation
y <- matrix(t(y[,2:9]), nrow = 4000, ncol = 1)
x1 <- matrix(t(x1[,2:9]), nrow = 4000, ncol = 1)
x2 <- matrix(x2, nrow = 9, ncol = 500)
x2 <- matrix(x2[2:9,], nrow = 4000, ncol = 1)

data<-data.frame(id, y,x1, x2)
Panel_data<-pdata.frame(data,index = c("id"))

# Models
m_p <- plm(y ~ x1 + x2, data = Panel_data, model = "pooling")
m_f1 <- plm(y ~ x1 + x2, data = Panel_data, model = "within")
m_f2 <- plm(y ~ lag(y, 1) + x1 + x2, data = Panel_data, model = "within")

m_gmm <- pgmm(y ~ lag(y, 1) + x1 + x2 | lag(y, 2:8), data = Panel_data, model = "twosteps")
summary(m_gmm)
# True values: lag(y): 0.5, x1: 0.5, x2: -1, 
stargazer(m_p, m_f1, m_f2,m_gmm, type="text", df=FALSE)  

############################ Exercise 3 ##################################
# Simulating dynamic panel data
  # Fixed effects
  # x1 and x2 are not autocorrelated

rm(list=ls())
set.seed(1234567) # setting seed

rho<-0.5 # Autoregressive term
# Generating ID
id<- c(t(kronecker(matrix(1,1,8),1:500)))

# Independent variables
fe <- rnorm(500,0,1) # fixed effect
fe2<-c(t(kronecker(matrix(1,1,9),fe)))
x1 <- (-1*fe2 + rnorm(4500,0,1))/2
x2 <- rnorm(4500,0,1)

Fit <- x1*0.5 -x2 + fe2 # XB
Fit <-matrix(Fit, nrow = 9, ncol = 500)
A <-diag(9) 
A[2:9,1:8] <- A[2:9,1:8]-rho*diag(8)
Mat <- inv(A)
SIG<- diag(9)
VAR<- Mat %*% SIG %*% t(Mat) # Variance

Means <- Mat %*% Fit # Means

y <- matrix(, nrow = 500, ncol = 9) # Dependent variable
for(i in 1:500){
  y[i,] <- rmvnorm(n=1, mean=t(Means[,i]), sigma=VAR)
}
# Some data transformation
y <- matrix(t(y[,2:9]), nrow = 4000, ncol = 1)
x1 <- matrix(x1, nrow = 9, ncol = 500)
x1 <- matrix(x1[2:9,], nrow = 4000, ncol = 1)
x2 <- matrix(x2, nrow = 9, ncol = 500)
x2 <- matrix(x2[2:9,], nrow = 4000, ncol = 1)

data<-data.frame(id, y,x1, x2)
Panel_data<-pdata.frame(data,index = c("id"))

# Models
m_p <- plm(y ~ x1 + x2, data = Panel_data, model = "pooling")
m_f1 <- plm(y ~ x1 + x2, data = Panel_data, model = "within")
m_f2 <- plm(y ~ lag(y, 1) + x1 + x2, data = Panel_data, model = "within")
m_gmm <- pgmm(y ~ lag(y, 1) + x1 + x2 | lag(y, 2:8), data = Panel_data, model = "twosteps")
summary(m_gmm)
# True values: lag(y): 0.5, x1: 0.5, x2: -1, 

stargazer(m_p, m_f1, m_f2,m_gmm, type="text", df=FALSE)  

############################ Exercise 4 ##################################
# Random effect + random trend

rm(list=ls())
set.seed(1234567) # setting seed

rho<-0.5 # Autoregressive term
# Generating ID
id<- c(t(kronecker(matrix(1,1,8),1:500)))

# Independent variables
re <- rnorm(500,0,1) # random effect
ret <- rnorm(500,0,3) # random trend
re2<-c(t(kronecker(matrix(1,1,9),re)))
trend <- 0:8
ret2<-matrix(t(ret%*%t(trend))/10, nrow = 4500, ncol = 1)

x1 <- rnorm(4500,0,2)
x2 <- rnorm(4500,0,1)

Fit <- x1*0.5 -x2 + re2 + ret2 # XB
Fit <-matrix(Fit, nrow = 9, ncol = 500)
A <-diag(9) 
A[2:9,1:8] <- A[2:9,1:8]-rho*diag(8)
Mat <- inv(A)
SIG<- diag(9)
VAR<- Mat %*% SIG %*% t(Mat) # Variance

Means <- Mat %*% Fit # Means

y <- matrix(, nrow = 500, ncol = 9) # Dependent variable
for(i in 1:500){
  y[i,] <- rmvnorm(n=1, mean=t(Means[,i]), sigma=VAR)
}
# Some data transformation
y <- matrix(t(y[,2:9]), nrow = 4000, ncol = 1)
x1 <- matrix(x1, nrow = 9, ncol = 500)
x1 <- matrix(x1[2:9,], nrow = 4000, ncol = 1)
x2 <- matrix(x2, nrow = 9, ncol = 500)
x2 <- matrix(x2[2:9,], nrow = 4000, ncol = 1)

data<-data.frame(id, y,x1, x2)
Panel_data<-pdata.frame(data,index = c("id"))

# Models
m_p <- plm(y ~ x1 + x2, data = Panel_data, model = "pooling")
m_f1 <- plm(y ~ x1 + x2, data = Panel_data, model = "within")
m_f2 <- plm(y ~ lag(y, 1) + x1 + x2, data = Panel_data, model = "within")
m_gmm <- pgmm(y ~ lag(y, 1) + x1 + x2 | lag(y, 2:8), data = Panel_data, model = "twosteps")
stargazer(m_p, m_f1, m_f2,m_gmm, type="text", df=FALSE)  
summary(m_gmm)